@@ -1,33 +1,9 @@
-use bstr::BStr;
-
 use crate::{
     immutable,
-    immutable::{parse, Blob, Commit, Tag, Tree},
-    Kind, Time,
+    immutable::{Blob, Commit, Tag, Tree},
+    Kind,
 };
 
-/// A signature is created by an actor at a certain time.
-///
-/// Note that this is not a cryptographical signature.
-#[derive(PartialEq, Eq, Debug, Hash, Ord, PartialOrd, Clone)]
-#[cfg_attr(feature = "serde1", derive(serde::Serialize, serde::Deserialize))]
-pub struct Signature<'a> {
-    /// The actor's name.
-    #[cfg_attr(feature = "serde1", serde(borrow))]
-    pub name: &'a BStr,
-    /// The actor's email.
-    pub email: &'a BStr,
-    /// The time stamp at which the signature was performed.
-    pub time: Time,
-}
-
-impl<'a> Signature<'a> {
-    /// Deserialize a signature from the given `data`.
-    pub fn from_bytes(data: &'a [u8]) -> Result<Signature<'a>, decode::Error> {
-        parse::signature(data).map(|(_, t)| t).map_err(decode::Error::from)
-    }
-}
-
 /// An immutable object representing [`Trees`][Tree], [`Blobs`][Blob], [`Commits`][Commit], or [`Tags`][Tag].
 #[derive(PartialEq, Eq, Debug, Hash, Ord, PartialOrd, Clone)]
 #[cfg_attr(feature = "serde1", derive(serde::Serialize, serde::Deserialize))]
@@ -210,66 +186,79 @@ mod convert {
 }
 
 ///
+#[cfg(feature = "verbose-object-parsing-errors")]
 pub mod decode {
-    use nom::error::ParseError;
-    use quick_error::quick_error;
+    use crate::bstr::{BString, ByteSlice};
 
-    quick_error! {
-        /// An error returned by various [`Commit`][crate::immutable::Commit] and [`Signature`][crate::immutable::Signature] methods
-        /// when they couldn't be decoded.
-        #[derive(Debug, Clone)]
-        #[allow(missing_docs)]
-        pub enum Error {
-            ParseIntegerError(msg: &'static str, number: crate::BString, err: btoi::ParseIntegerError) {
-                display("{}: {:?}", msg, number)
-                source(err)
-            }
-            Nom(err_msg: String) {
-                display("{}", err_msg)
-            }
-            NomDetail(input: crate::BString, msg: &'static str) {
-                display("{}: '{}' could not be parsed", msg, input)
-            }
-            ParseKindError(err: crate::types::Error) {
-                display("{}", err)
-                source(err)
-            }
-            ObjectKind(err: crate::Error) {
-                from()
-                source(err)
+    /// The type to be used for parse errors.
+    pub type ParseError<'a> = nom::error::VerboseError<&'a [u8]>;
+    /// The owned type to be used for parse errors.
+    pub type ParseErrorOwned = nom::error::VerboseError<BString>;
+
+    /// A type to indicate errors during parsing and to abstract away details related to `nom`.
+    #[derive(Debug, Clone)]
+    pub struct Error {
+        /// The actual error
+        pub inner: ParseErrorOwned,
+    }
+
+    impl<'a> From<nom::Err<ParseError<'a>>> for Error {
+        fn from(v: nom::Err<ParseError<'a>>) -> Self {
+            Error {
+                inner: match v {
+                    nom::Err::Error(err) | nom::Err::Failure(err) => nom::error::VerboseError {
+                        errors: err
+                            .errors
+                            .into_iter()
+                            .map(|(i, v)| (i.as_bstr().to_owned(), v))
+                            .collect(),
+                    },
+                    nom::Err::Incomplete(_) => unreachable!("we don't have streaming parsers"),
+                },
             }
         }
     }
 
-    impl Error {
-        fn set_parse_context(mut self, ctx: &'static str) -> Self {
-            if let Error::NomDetail(_, ref mut message) = self {
-                *message = ctx
-            }
-            self
-        }
-
-        pub(crate) fn context(msg: &'static str) -> impl Fn(nom::Err<Self>) -> nom::Err<Self> {
-            move |e: nom::Err<Self>| e.map(|e| e.set_parse_context(msg))
+    impl std::fmt::Display for Error {
+        fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+            self.inner.fmt(f)
         }
     }
 
-    impl ParseError<&[u8]> for Error {
-        fn from_error_kind(input: &[u8], _kind: nom::error::ErrorKind) -> Self {
-            Error::NomDetail(input.into(), "parse error")
-        }
+    impl std::error::Error for Error {}
+}
+
+///
+#[cfg(not(feature = "verbose-object-parsing-errors"))]
+pub mod decode {
+    /// The type to be used for parse errors, discards everything and is zero size
+    pub type ParseError<'a> = ();
+    /// The owned type to be used for parse errors, discards everything and is zero size
+    pub type ParseErrorOwned = ();
+
+    /// A type to indicate errors during parsing and to abstract away details related to `nom`.
+    #[derive(Debug, Clone)]
+    pub struct Error {
+        /// The actual error
+        pub inner: ParseErrorOwned,
+    }
 
-        fn append(_: &[u8], _: nom::error::ErrorKind, other: Self) -> Self {
-            other
+    impl<'a> From<nom::Err<ParseError<'a>>> for Error {
+        fn from(v: nom::Err<ParseError<'a>>) -> Self {
+            Error {
+                inner: match v {
+                    nom::Err::Error(err) | nom::Err::Failure(err) => err,
+                    nom::Err::Incomplete(_) => unreachable!("we don't have streaming parsers"),
+                },
+            }
         }
     }
 
-    impl From<nom::Err<Error>> for Error {
-        fn from(e: nom::Err<Error>) -> Self {
-            match e {
-                nom::Err::Error(err) | nom::Err::Failure(err) => Error::Nom(err.to_string()),
-                nom::Err::Incomplete(_) => unreachable!("we do not implement streaming parsers"),
-            }
+    impl std::fmt::Display for Error {
+        fn fmt(&self, _f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+            Ok(())
         }
     }
+
+    impl std::error::Error for Error {}
 }
